from typing import Any, Dict, Optional

from flair.data import Dictionary
from flair.models import TextPairClassifier

from embeddings.data.io import T_path
from embeddings.embedding.flair_embedding import FlairEmbedding
from embeddings.task.flair_task.text_classification import TextClassification


class TextPairClassification(TextClassification):
    def __init__(
        self,
        output_path: T_path,
        task_model_kwargs: Optional[Dict[str, Any]] = None,
        task_train_kwargs: Optional[Dict[str, Any]] = None,
    ):
        super().__init__(output_path, task_model_kwargs, task_train_kwargs)

    def build_task_model(self, embedding: FlairEmbedding, y_dictionary: Dictionary) -> None:
        self.model = TextPairClassifier(
            document_embeddings=embedding.model,
            label_dictionary=y_dictionary,
            **self.task_model_kwargs
        )



import abc
from abc import ABC
from typing import Generic, TypeVar

from embeddings.data.dataset import Dataset

Input = TypeVar("Input")
Output = TypeVar("Output")


class DataLoader(ABC, Generic[Input, Output]):
    @abc.abstractmethod
    def load(self, dataset: Dataset[Input]) -> Output:
        pass



import warnings
from pathlib import Path
from typing import Tuple

import datasets
import flair
from flair.datasets import CSVClassificationDataset, DataPairDataset

from embeddings.defaults import DATASET_PATH
from embeddings.transformation.flair_transformation.classification_corpus_transformation import (
    ClassificationCorpusTransformation,
)


class PairClassificationCorpusTransformation(ClassificationCorpusTransformation):
    def __init__(
        self,
        input_columns_names_pair: Tuple[str, str],
        target_column_name: str,
        datasets_path: Path = DATASET_PATH,
    ):
        super().__init__(input_columns_names_pair[0], target_column_name, datasets_path)
        self.pair_column_name: str = input_columns_names_pair[1]

    def _preprocess_subset(
        self, hf_datadict: datasets.DatasetDict, subset_name: str, output_path: Path
    ) -> DataPairDataset:
        csv_path = output_path.joinpath(f"{subset_name}.csv")
        self._save_to_csv(hf_datadict, subset_name, csv_path)

        if flair.__version__ != "0.8":
            warnings.warn(
                f"Implementation of {type(self).__name__} could be deprecated due to "
                f"new flair release.",
                DeprecationWarning,
            )
        # todo: change that function call for _to_csv_classification_dataset after a new flair
        #  release for code coherence
        return self._to_data_pair_dataset(hf_datadict, subset_name, output_path)

    def _to_data_pair_dataset(
        self, hf_datadict: datasets.DatasetDict, subset_name: str, output_path: Path
    ) -> DataPairDataset:
        columns = [
            hf_datadict[subset_name].column_names.index(self.input_column_name),
            hf_datadict[subset_name].column_names.index(self.pair_column_name),
            hf_datadict[subset_name].column_names.index(self.target_column_name),
        ]

        return DataPairDataset(
            output_path.joinpath(f"{subset_name}.csv"), columns=columns, separator=","
        )

    def _to_csv_classification_dataset(
        self, hf_datadict: datasets.DatasetDict, subset_name: str, output_path: Path
    ) -> CSVClassificationDataset:
        if flair.__version__ == "0.8":
            raise NotImplementedError(
                f"Usage of {CSVClassificationDataset.__name__} for pair "
                f"classification is not possible for flair 0.8."
            )

        column_name_map = {
            hf_datadict[subset_name].column_names.index(self.input_column_name): "text",
            hf_datadict[subset_name].column_names.index(self.pair_column_name): "pair",
            hf_datadict[subset_name].column_names.index(self.target_column_name): "label",
        }

        return CSVClassificationDataset(output_path.joinpath(f"{subset_name}.csv"), column_name_map)

    def _check_compatibility(self, dataset: datasets.Dataset) -> None:
        super()._check_compatibility(dataset)
        self._check_column_in_dataset(dataset, self.pair_column_name)



from typing import Dict

import numpy as np
from flair.data import Corpus

from embeddings.embedding.flair_embedding import FlairEmbedding
from embeddings.model.model import Model
from embeddings.task.flair_task.flair_task import FlairTask


class FlairModel(Model[Corpus, Dict[str, np.ndarray]]):
    def __init__(self, embedding: FlairEmbedding, task: FlairTask) -> None:
        super().__init__()
        self.embedding = embedding
        self.task = task

    def execute(self, data: Corpus) -> Dict[str, np.ndarray]:
        self.task.build_task_model(
            embedding=self.embedding, y_dictionary=self.task.make_y_dictionary(data)
        )
        return self.task.fit_predict(data)



from typing import Any, Dict, List, Optional, Union

import datasets
import numpy as np
import torch

from embeddings.metric.metric import Metric

HF_metric_input = Union[List[Any], np.ndarray, torch.Tensor]


class HuggingFaceMetric(Metric[HF_metric_input, Dict[Any, Any]]):
    def __init__(
        self, name: str, compute_kwargs: Optional[Dict[str, Any]] = None, **init_kwargs: Any
    ):
        super().__init__(name)
        if init_kwargs.get("process_id", 0) != 0:
            raise ValueError(
                "Metric computation should be run on the main process. "
                "Otherwise it would not return results in dict."
            )

        self.metric = datasets.load_metric(name, **init_kwargs)
        self.compute_kwargs = {} if compute_kwargs is None else compute_kwargs

    def compute(
        self, y_true: Optional[HF_metric_input], y_pred: Optional[HF_metric_input], **kwargs: Any
    ) -> Dict[Any, Any]:
        result = self.metric.compute(
            references=y_true, predictions=y_pred, **self.compute_kwargs, **kwargs
        )
        assert isinstance(result, Dict)
        return result

    def __str__(self) -> str:
        compute_kwargs_str = "__".join(f"{k}_{v}" for k, v in self.compute_kwargs.items())
        if compute_kwargs_str:
            return f"{super().__str__()}__{compute_kwargs_str}"
        else:
            return super().__str__()



import abc
from typing import Any, Dict, List, Sequence, Union

import numpy as np
import torch

from embeddings.evaluator.evaluator import Evaluator
from embeddings.metric.metric import Metric


class MetricsEvaluator(Evaluator[Dict[str, np.ndarray], Dict[str, Any]]):
    @property
    @abc.abstractmethod
    def metrics(
        self,
    ) -> Sequence[Metric[Union[List[Any], np.ndarray, torch.Tensor], Dict[Any, Any]]]:
        pass

    def evaluate(self, data: Dict[str, np.ndarray]) -> Dict[str, Any]:
        return {
            str(metric): metric.compute(y_true=data["y_true"], y_pred=data["y_pred"])
            for metric in self.metrics
        }



from typing import Any, Dict

import srsly

from embeddings.utils.results_persister import ResultsPersister


class JsonPersister(ResultsPersister[Dict[str, Any]]):
    def __init__(self, path: str):
        self.path = path

    def persist(self, result: Dict[str, Any], **kwargs: Any) -> None:
        srsly.write_json(self.path, result)



from itertools import chain
from typing import Any, Dict, List

import numpy as np

from embeddings.metric.hugging_face_metric import HuggingFaceMetric


class UnitSeqevalMetric(HuggingFaceMetric):
    ALLOWED_TAG_PREFIXES = {"U-"}

    def __init__(self) -> None:
        super().__init__(
            "seqeval", compute_kwargs={"suffix": None, "scheme": "BILOU", "mode": "strict"}
        )

    @staticmethod
    def _convert_single_tag_to_bilou_scheme(tags: np.ndarray) -> List[List[str]]:
        return [[f"U-{tag}" for tag in sequence] for sequence in tags]

    @staticmethod
    def _have_tags_unit_prefix(tags: np.ndarray) -> bool:
        unique_tags = set(chain.from_iterable(tags))
        tag_prefixes = set(it[0:2] for it in unique_tags)
        return not tag_prefixes.difference(UnitSeqevalMetric.ALLOWED_TAG_PREFIXES)

    def compute(self, y_true: np.ndarray, y_pred: np.ndarray, **kwargs: Any) -> Dict[str, Any]:
        if not self._have_tags_unit_prefix(y_pred):
            y_pred = self._convert_single_tag_to_bilou_scheme(y_pred)
        if not self._have_tags_unit_prefix(y_true):
            y_true = self._convert_single_tag_to_bilou_scheme(y_true)

        return super().compute(y_true=y_true, y_pred=y_pred, **kwargs)

    def __str__(self) -> str:
        return "UnitSeqeval"



from abc import ABC
from typing import Generic, Optional, TypeVar

from embeddings.data.data_loader import DataLoader
from embeddings.data.dataset import Dataset
from embeddings.evaluator.evaluator import Evaluator
from embeddings.model.model import Model
from embeddings.pipeline.pipeline import Pipeline
from embeddings.pipeline.standard_pipeline import StandardPipeline
from embeddings.transformation.transformation import Transformation

Data = TypeVar("Data")
CreationData = TypeVar("CreationData")
LoaderResult = TypeVar("LoaderResult")
CreationLoaderResult = TypeVar("CreationLoaderResult")
TransformationResult = TypeVar("TransformationResult")
CreationTransformationResult = TypeVar("CreationTransformationResult")
ModelResult = TypeVar("ModelResult")
CreationModelResult = TypeVar("CreationModelResult")
EvaluationResult = TypeVar("EvaluationResult")
CreationEvaluationResult = TypeVar("CreationEvaluationResult")


class PipelineBuilder(
    ABC,
    Generic[Data, LoaderResult, TransformationResult, ModelResult, EvaluationResult],
):
    def __init__(
        self,
        dataset: Optional[Dataset[Data]] = None,
        data_loader: Optional[DataLoader[Data, LoaderResult]] = None,
        transformation: Optional[Transformation[LoaderResult, TransformationResult]] = None,
        model: Optional[Model[TransformationResult, ModelResult]] = None,
        evaluator: Optional[Evaluator[ModelResult, EvaluationResult]] = None,
    ) -> None:
        self.dataset = dataset
        self.data_loader = data_loader
        self.transformation = transformation
        self.model = model
        self.evaluator = evaluator

    @staticmethod
    def with_dataset(
        dataset: Dataset[CreationData],
    ) -> "PipelineBuilder[CreationData, LoaderResult, TransformationResult, ModelResult, EvaluationResult]":
        return PipelineBuilder(dataset)

    def with_loader(
        self: "PipelineBuilder[Data, LoaderResult, TransformationResult, ModelResult, EvaluationResult]",
        data_loader: DataLoader[Data, CreationLoaderResult],
    ) -> "PipelineBuilder[Data, CreationLoaderResult, TransformationResult, ModelResult, EvaluationResult]":
        return PipelineBuilder(self.dataset, data_loader, None, self.model, self.evaluator)

    def with_transformation(
        self: "PipelineBuilder[Data, LoaderResult, TransformationResult, ModelResult, EvaluationResult]",
        transformation: Transformation[LoaderResult, CreationTransformationResult],
    ) -> "PipelineBuilder[Data, LoaderResult, CreationTransformationResult, ModelResult, EvaluationResult]":
        return PipelineBuilder(self.dataset, self.data_loader, transformation, None, self.evaluator)

    def with_model(
        self: "PipelineBuilder[Data, LoaderResult, TransformationResult, ModelResult, EvaluationResult]",
        model: Model[TransformationResult, CreationModelResult],
    ) -> "PipelineBuilder[Data, LoaderResult, TransformationResult, CreationModelResult, EvaluationResult]":
        return PipelineBuilder(self.dataset, self.data_loader, self.transformation, model, None)

    def with_evaluator(
        self: "PipelineBuilder[Data, LoaderResult, TransformationResult, ModelResult, EvaluationResult]",
        evaluator: Evaluator[ModelResult, CreationEvaluationResult],
    ) -> "PipelineBuilder[Data, LoaderResult, TransformationResult, ModelResult, CreationEvaluationResult]":
        return PipelineBuilder(
            self.dataset, self.data_loader, self.transformation, self.model, evaluator
        )

    def build(
        self: "PipelineBuilder[Data, LoaderResult, TransformationResult, ModelResult, EvaluationResult]",
    ) -> Pipeline[EvaluationResult]:
        assert self.dataset is not None
        assert self.data_loader is not None
        assert self.transformation is not None
        assert self.model is not None
        assert self.evaluator is not None
        return StandardPipeline(
            self.dataset,
            self.data_loader,
            self.transformation,
            self.model,
            self.evaluator,
        )



from typing import Generic, TypeVar

from embeddings.embedding.embedding import Embedding
from embeddings.model.model import Model
from embeddings.task.task import Task

Input = TypeVar("Input")
EmbeddingResult = TypeVar("EmbeddingResult")
Output = TypeVar("Output")


class BaseModel(Model[Input, Output], Generic[Input, EmbeddingResult, Output]):
    def __init__(
        self, embedding: Embedding[Input, EmbeddingResult], task: Task[EmbeddingResult, Output]
    ) -> None:
        self.embedding = embedding
        self.task = task

    def execute(self, data: Input) -> Output:
        embedded = self.embedding.embed(data)
        return self.task.fit_predict(embedded)



import abc
from abc import ABC
from typing import Generic, TypeVar

from embeddings.utils.results_persister import ResultsPersister

Input = TypeVar("Input")
Output = TypeVar("Output")


class Evaluator(ABC, Generic[Input, Output]):
    def __init__(self) -> None:
        pass

    @abc.abstractmethod
    def evaluate(self, data: Input) -> Output:
        pass

    def persisting(self, persister: ResultsPersister[Output]) -> "Evaluator[Input, Output]":
        return PersistingEvaluator(self, persister)


class PersistingEvaluator(Evaluator[Input, Output]):
    def __init__(
        self, base_evaluator: Evaluator[Input, Output], persister: ResultsPersister[Output]
    ):
        self.base_evaluator = base_evaluator
        self.persister = persister

    def evaluate(self, data: Input) -> Output:
        result = self.base_evaluator.evaluate(data)
        self.persister.persist(result)
        return result



from abc import ABC
from dataclasses import dataclass, field
from typing import Any, Literal, Optional

from flair.embeddings import FastTextEmbeddings

from embeddings.embedding.flair_embedding import FlairEmbedding
from embeddings.embedding.static.config import SingleFileConfig, StaticModelHubConfig
from embeddings.embedding.static.word import StaticWordEmbedding


@dataclass
class KGR10FastTextConfig(SingleFileConfig):
    method: Optional[Literal["cbow", "skipgram"]] = None
    dimension: Optional[Literal[100, 300]] = None
    model_name: str = field(default=f"kgr10.plain.{method}.dim{dimension}.neg10.bin", init=False)
    repo_id: str = field(default="clarin-pl/fastText-kgr10", init=False)

    def __post_init__(self) -> None:
        if not self.method:
            self.method = self.default_config["method"]

        if not self.dimension:
            self.dimension = self.default_config["dimension"]

        self.model_name = f"kgr10.plain.{self.method}.dim{self.dimension}.neg10.bin"


class SingleFileEmbedding(StaticWordEmbedding, ABC):
    def __init__(self, config: SingleFileConfig, **load_model_kwargs: Any):
        super().__init__(config.cached_model, **load_model_kwargs)
        self.config = config


class FlairFastTextEmbedding(FlairEmbedding):
    def _get_model(self) -> FastTextEmbeddings:
        return FastTextEmbeddings(self.name, **self.load_model_kwargs)


class KGR10FastTextEmbedding(SingleFileEmbedding, FlairFastTextEmbedding):
    @staticmethod
    def from_config(config: StaticModelHubConfig, **kwargs: Any) -> "KGR10FastTextEmbedding":
        if not isinstance(config, KGR10FastTextConfig):
            raise ValueError(f"Wrong config type {type(config)}, expected {KGR10FastTextConfig}.")
        return KGR10FastTextEmbedding(config)

    @staticmethod
    def from_default_config(
        config: StaticModelHubConfig, **kwargs: Any
    ) -> "KGR10FastTextEmbedding":
        config = KGR10FastTextEmbedding.get_config(**config.default_config)
        return KGR10FastTextEmbedding(config)

    @staticmethod
    def get_config(**kwargs: Any) -> KGR10FastTextConfig:
        return KGR10FastTextConfig(**kwargs)



import abc
from abc import ABC
from typing import Any, Generic, TypeVar

Input = TypeVar("Input")


class ResultsPersister(ABC, Generic[Input]):
    @abc.abstractmethod
    def persist(self, result: Input, **kwargs: Any) -> None:
        pass



from typing import Any, Dict, List, Optional, Sequence, Set, Union

import numpy as np
import torch

from embeddings.evaluator.metrics_evaluator import MetricsEvaluator
from embeddings.metric.hugging_face_metric import HuggingFaceMetric
from embeddings.metric.metric import Metric
from embeddings.metric.unit_seqeval_metric import UnitSeqevalMetric


class SequenceLabelingEvaluator(MetricsEvaluator):
    SEQEVAL_EVALUATION_MODES: Set[str] = {"conll", "strict"}

    def __init__(
        self, evaluation_mode: str = "conll", tagging_scheme: Optional[str] = None
    ) -> None:
        super().__init__()
        self.metric = self._get_metric(evaluation_mode, tagging_scheme)

    def _get_metric(
        self, evaluation_mode: str, tagging_scheme: Optional[str] = None
    ) -> Union[HuggingFaceMetric, UnitSeqevalMetric]:
        if evaluation_mode in SequenceLabelingEvaluator.SEQEVAL_EVALUATION_MODES:
            if evaluation_mode == "strict" and not tagging_scheme:
                raise ValueError("Tagging scheme must be set, when using strict evaluation mode!")
            return HuggingFaceMetric(
                name="seqeval",
                compute_kwargs={
                    "mode": evaluation_mode if evaluation_mode == "strict" else None,
                    "scheme": tagging_scheme,
                },
            )
        elif evaluation_mode == "unit":
            return UnitSeqevalMetric()
        else:
            raise ValueError(
                f"Evaluation mode {evaluation_mode} not supported. Must be one of [unit, conll, strict]."
            )

    @property
    def metrics(
        self,
    ) -> Sequence[Metric[Union[List[Any], np.ndarray, torch.Tensor], Dict[Any, Any]]]:
        return [self.metric]



from pathlib import Path

import datasets
from flair.datasets import CSVClassificationDataset

from embeddings.transformation.flair_transformation.corpus_transformation import (
    CorpusTransformation,
)


class ClassificationCorpusTransformation(CorpusTransformation):
    def _preprocess_subset(
        self, hf_datadict: datasets.DatasetDict, subset_name: str, output_path: Path
    ) -> CSVClassificationDataset:
        csv_path = output_path.joinpath(f"{subset_name}.csv")
        self._save_to_csv(hf_datadict, subset_name, csv_path)
        column_name_map = {
            hf_datadict[subset_name].column_names.index(self.input_column_name): "text",
            hf_datadict[subset_name].column_names.index(self.target_column_name): "label",
        }
        return CSVClassificationDataset(csv_path, column_name_map)

    def _save_to_csv(
        self, hf_datadict: datasets.DatasetDict, subset_name: str, csv_path: Path
    ) -> None:
        label_map = hf_datadict[subset_name].features[self.target_column_name].names
        hf_datadict[subset_name] = hf_datadict[subset_name].map(
            lambda row: {self.target_column_name: label_map[row[self.target_column_name]]},
        )
        hf_datadict[subset_name].to_csv(csv_path, header=False, index=False)

    def _check_task(self, dataset: datasets.Dataset) -> None:
        if not isinstance(dataset.features[self.target_column_name], datasets.ClassLabel):
            raise ValueError(f"Type of target column is not '{datasets.ClassLabel.__name__}'.")



import logging
import os
from typing import Union

DEFAULT_LOG_LEVEL = os.getenv("LOG_LEVEL", logging.INFO)


def get_logger(name: str, log_level: Union[str, int] = DEFAULT_LOG_LEVEL) -> logging.Logger:
    log_level = log_level
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    handler = logging.StreamHandler()
    handler.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(log_level)
    logger.addHandler(handler)
    logger.propagate = False

    return logger



from dataclasses import dataclass
from typing import Any

import srsly
from huggingface_hub import cached_download, hf_hub_url
from requests import HTTPError


@dataclass
class StaticModelHubConfig:
    repo_id: str

    @property
    def model_type_reference(self) -> str:
        if isinstance(reference := self._load_hub_json("module.json")["type"], str):
            return reference
        else:
            raise ValueError(f"Wrong format of import reference {reference}.")

    @property
    def default_config(self) -> dict[str, Any]:
        if isinstance(config := self._load_hub_json("default_config.json"), dict):
            return config
        else:
            raise ValueError(f"Wrong format of default config {config}.")

    def _load_hub_json(self, filename: str) -> Any:
        url = self._get_file_hf_hub_url(filename)
        try:
            path = cached_download(url)
        except HTTPError:
            raise EnvironmentError(
                "Repository not found or wrong format of a given model (module.json not found)."
            )
        return srsly.read_json(path)

    def _get_file_hf_hub_url(self, filename: str) -> str:
        url: str = hf_hub_url(self.repo_id, filename=filename)
        return url


@dataclass
class SingleFileConfig(StaticModelHubConfig):
    model_name: str

    @property
    def cached_model(self) -> str:
        url: str = self._get_file_hf_hub_url(self.model_name)
        path: str = cached_download(url)
        return path



from typing import Any, Dict, Optional

import datasets
import numpy as np
from flair.data import Corpus

from embeddings.data.hugging_face_data_loader import HuggingFaceDataLoader
from embeddings.data.hugging_face_dataset import HuggingFaceDataset
from embeddings.data.io import T_path
from embeddings.embedding.flair_embedding import FlairTransformerWordEmbedding
from embeddings.evaluator.sequence_labeling_evaluator import SequenceLabelingEvaluator
from embeddings.model.flair_model import FlairModel
from embeddings.pipeline.standard_pipeline import StandardPipeline
from embeddings.task.flair_task.sequence_labeling import SequenceLabeling
from embeddings.transformation.flair_transformation.column_corpus_transformation import (
    ColumnCorpusTransformation,
)


class HuggingFaceSequenceLabelingPipeline(
    StandardPipeline[str, datasets.DatasetDict, Corpus, Dict[str, np.ndarray], Dict[str, Any]]
):
    def __init__(
        self,
        embedding_name: str,
        dataset_name: str,
        input_column_name: str,
        target_column_name: str,
        output_path: T_path,
        hidden_size: int,
        evaluation_mode: str = "conll",
        tagging_scheme: Optional[str] = None,
        task_model_kwargs: Optional[Dict[str, Any]] = None,
        task_train_kwargs: Optional[Dict[str, Any]] = None,
    ):
        dataset = HuggingFaceDataset(dataset_name)
        data_loader = HuggingFaceDataLoader()
        transformation = ColumnCorpusTransformation(input_column_name, target_column_name)
        embedding = FlairTransformerWordEmbedding(embedding_name)
        task = SequenceLabeling(
            output_path,
            hidden_size=hidden_size,
            task_model_kwargs=task_model_kwargs,
            task_train_kwargs=task_train_kwargs,
        )
        model = FlairModel(embedding, task)
        evaluator = SequenceLabelingEvaluator(
            evaluation_mode=evaluation_mode, tagging_scheme=tagging_scheme
        )
        super().__init__(dataset, data_loader, transformation, model, evaluator)



from abc import ABC, abstractmethod
from typing import Any, Type

from embeddings.embedding.flair_embedding import FlairEmbedding
from embeddings.embedding.static.config import StaticModelHubConfig
from embeddings.utils.utils import import_from_string


class StaticWordEmbedding(FlairEmbedding, ABC):
    @staticmethod
    @abstractmethod
    def get_config(**kwargs: Any) -> StaticModelHubConfig:
        pass

    @staticmethod
    @abstractmethod
    def from_default_config(config: StaticModelHubConfig, **kwargs: Any) -> "StaticWordEmbedding":
        pass

    @staticmethod
    @abstractmethod
    def from_config(config: StaticModelHubConfig, **kwargs: Any) -> "StaticWordEmbedding":
        pass


class AutoStaticWordEmbedding:
    @staticmethod
    def from_default_config(repo_id: str, **kwargs: Any) -> StaticWordEmbedding:
        """Returns a static word embedding model initialised using the default_config.json file,
        that should be placed in the given repository.
        Example: https://huggingface.co/clarin-pl/fastText-kgr10/blob/main/default_config.json"""
        config = StaticModelHubConfig(repo_id)
        return AutoStaticWordEmbedding._get_model_cls(config).from_default_config(config, **kwargs)

    @staticmethod
    def from_config(config: StaticModelHubConfig, **kwargs: Any) -> StaticWordEmbedding:
        """Returns a static word embedding model initialised using the provided config."""
        return AutoStaticWordEmbedding._get_model_cls(config).from_config(config, **kwargs)

    @staticmethod
    def _get_model_cls(config: StaticModelHubConfig) -> Type[StaticWordEmbedding]:
        """Return a class of a static word embedding model. The class should be specified in the
        module.json file, that should be placed in the given repository.
        Example: https://huggingface.co/clarin-pl/fastText-kgr10/blob/main/module.json"""
        cls: Type[StaticWordEmbedding] = import_from_string(config.model_type_reference)
        if not issubclass(cls, StaticWordEmbedding):
            raise ValueError(f"Type reference has to be subclass of {StaticWordEmbedding}.")
        return cls



import abc
from abc import ABC
from typing import Any, List

from flair.data import Sentence
from flair.embeddings import TransformerDocumentEmbeddings, TransformerWordEmbeddings
from flair.embeddings.base import Embeddings

from embeddings.embedding.embedding import Embedding


class FlairEmbedding(Embedding[List[Sentence], List[Sentence]]):
    def __init__(self, name: str, **load_model_kwargs: Any) -> None:
        super().__init__()
        self.load_model_kwargs = load_model_kwargs
        self.name = name
        self.model = self._get_model()

    def embed(self, data: List[Sentence]) -> List[Sentence]:
        self.model.embed(data)
        return data

    @abc.abstractmethod
    def _get_model(self) -> Embeddings:
        pass


class FlairTransformerEmbedding(FlairEmbedding, ABC):
    def __init__(self, name: str, fine_tune: bool = False, **load_model_kwargs: Any) -> None:
        load_model_kwargs["fine_tune"] = fine_tune
        super().__init__(name, **load_model_kwargs)


class FlairTransformerWordEmbedding(FlairTransformerEmbedding):
    def _get_model(self) -> TransformerWordEmbeddings:
        return TransformerWordEmbeddings(self.name, **self.load_model_kwargs)


class FlairTransformerDocumentEmbedding(FlairTransformerEmbedding):
    def _get_model(self) -> TransformerDocumentEmbeddings:
        return TransformerDocumentEmbeddings(self.name, **self.load_model_kwargs)



from typing import Any

from embeddings.embedding.flair_embedding import FlairEmbedding, FlairTransformerWordEmbedding
from embeddings.embedding.static.word import AutoStaticWordEmbedding


class AutoFlairWordEmbedding:
    @staticmethod
    def from_hub(repo_id: str, **kwargs: Any) -> FlairEmbedding:
        """Loads an embedding model from hugging face hub, if the model is compatible with
        Transformers or the current library.

        In case of StaticWordEmbedding, a default config is used during initialisation. If you
        rather want to specify custom config, use the AutoStaticWordEmbedding.from_config function.
        """

        try:
            return FlairTransformerWordEmbedding(repo_id, **kwargs)
        except EnvironmentError:
            return AutoStaticWordEmbedding.from_default_config(repo_id, **kwargs)



import importlib
from typing import Any


def import_from_string(dotted_path: str) -> Any:
    """
    The function taken from github.com/UKPLab/sentence-transformers.

    Import a dotted module path and return the attribute/class designated by the
    last name in the path. Raise ImportError if the import failed.
    """
    try:
        module_path, class_name = dotted_path.rsplit(".", 1)
    except ValueError:
        msg = "%s doesn't look like a module path" % dotted_path
        raise ImportError(msg)

    try:
        module = importlib.import_module(dotted_path)
    except:
        module = importlib.import_module(module_path)

    try:
        return getattr(module, class_name)
    except AttributeError:
        msg = 'Module "%s" does not define a "%s" attribute/class' % (module_path, class_name)
        raise ImportError(msg)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['embeddings',
 'embeddings.data',
 'embeddings.embedding',
 'embeddings.embedding.static',
 'embeddings.evaluator',
 'embeddings.metric',
 'embeddings.model',
 'embeddings.pipeline',
 'embeddings.task',
 'embeddings.task.flair_task',
 'embeddings.transformation',
 'embeddings.transformation.flair_transformation',
 'embeddings.utils',
 'experimental',
 'experimental.datasets',
 'experimental.datasets.utils',
 'experimental.embeddings',
 'experimental.embeddings.language_models',
 'experimental.embeddings.scripts']

package_data = \
{'': ['*']}

install_requires = \
['Pillow==8.2.0',
 'datasets==1.6.1',
 'flair==0.8.0',
 'numpy==1.19.5',
 'requests==2.25.1',
 'scikit-learn==0.24.1',
 'seqeval==1.2.2',
 'spacy==3.1.0',
 'srsly==2.4.1',
 'tensorboard==2.4.1',
 'torch==1.9.0',
 'transformers==4.8.2',
 'typer==0.3.2']

setup_kwargs = {
    'name': 'clarinpl-embeddings',
    'version': '0.0.1rc28',
    'description': '',
    'long_description': None,
    'author': 'Roman Bartusiak',
    'author_email': 'riomus@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/CLARIN-PL/embeddings',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '==3.9.6',
}


setup(**setup_kwargs)



from pathlib import Path
from typing import Any, Union

from embeddings.data.dataset import Dataset


class HuggingFaceDataset(Dataset[str]):
    def __init__(self, dataset: Union[str, Path], **load_dataset_kwargs: Any):
        super().__init__()
        self.dataset = dataset
        self.load_dataset_kwargs = load_dataset_kwargs



from typing import Generic, TypeVar

from embeddings.data.data_loader import DataLoader
from embeddings.data.dataset import Dataset
from embeddings.evaluator.evaluator import Evaluator
from embeddings.model.model import Model
from embeddings.pipeline.pipeline import Pipeline
from embeddings.transformation.transformation import Transformation

EvaluationResult = TypeVar("EvaluationResult")
Data = TypeVar("Data")
LoaderResult = TypeVar("LoaderResult")
TransformationResult = TypeVar("TransformationResult")
ModelResult = TypeVar("ModelResult")


class StandardPipeline(
    Pipeline[EvaluationResult],
    Generic[Data, LoaderResult, TransformationResult, ModelResult, EvaluationResult],
):
    def __init__(
        self,
        dataset: Dataset[Data],
        data_loader: DataLoader[Data, LoaderResult],
        transformation: Transformation[LoaderResult, TransformationResult],
        model: Model[TransformationResult, ModelResult],
        evaluator: Evaluator[ModelResult, EvaluationResult],
    ) -> None:
        self.dataset = dataset
        self.data_loader = data_loader
        self.transformation = transformation
        self.model = model
        self.evaluator = evaluator

    def run(self) -> EvaluationResult:
        loaded_data = self.data_loader.load(self.dataset)
        transformed_data = self.transformation.transform(loaded_data)
        model_result = self.model.execute(transformed_data)
        return self.evaluator.evaluate(model_result)
